"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRemoveTmpDirectoryTask = exports.createGenerateTmpDirectoryTask = exports.createExecCommandTasks = exports.createInstallAppTasks = exports.createParseFileTask = exports.createDecryptFileTask = exports.createLoadFileTask = exports.createFileFormatVerifyTask = void 0;
const chalk_1 = __importDefault(require("chalk"));
const rimraf_1 = __importDefault(require("rimraf"));
const path_1 = require("path");
const util_1 = require("util");
const utils_1 = require("./utils");
const lib_1 = require("../lib");
function createFileFormatVerifyTask(filePath) {
    return {
        title: 'Verify config file format',
        task: () => {
            if (!(0, utils_1.isPackFile)(filePath)) {
                throw new Error(chalk_1.default.red('The file format is incorrect, please use the .pack file format.'));
            }
        }
    };
}
exports.createFileFormatVerifyTask = createFileFormatVerifyTask;
function createLoadFileTask(filePath) {
    const fileName = (0, path_1.basename)(filePath);
    return {
        title: `Loading ${fileName} file`,
        task: async (ctx) => {
            try {
                ctx.text = await (0, utils_1.loadFile)(filePath);
            }
            catch (error) {
                throw new Error(chalk_1.default.red(error.message));
            }
        }
    };
}
exports.createLoadFileTask = createLoadFileTask;
function createDecryptFileTask(filePath, password) {
    const fileName = (0, path_1.basename)(filePath);
    return {
        title: `Decrypt ${fileName} file`,
        skip: (ctx) => !(0, utils_1.isHashCode)(ctx.text),
        task: (ctx) => {
            try {
                ctx.text = (0, utils_1.decrypt)(ctx.text, password);
            }
            catch (error) {
                throw new Error(chalk_1.default.red('Incorrect password'));
            }
        }
    };
}
exports.createDecryptFileTask = createDecryptFileTask;
function createParseFileTask(filePath) {
    const fileName = (0, path_1.basename)(filePath);
    return {
        title: `Parsing ${fileName} file`,
        task: (ctx) => {
            const result = (0, utils_1.parse)(ctx.text);
            ctx.tasks = result && Array.isArray(result) ? result : [];
        }
    };
}
exports.createParseFileTask = createParseFileTask;
function createInstallAppTasks(ctx) {
    return [
        createCommandLineToolsTasks(),
        ...ctx.tasks
            .filter((task) => (0, utils_1.isAppType)(task))
            .map((app) => {
            return {
                title: app.alias || app.name,
                task: async (_, task) => {
                    if (await (0, lib_1.isInstalled)(app.name)) {
                        return [];
                    }
                    let actions = [createDownloadTask(app)];
                    if (app.action && app.action.includes('install')) {
                        actions = [...actions, createInstallTask(app)];
                    }
                    if (app.action && hasActiveAction(app.action)) {
                        actions = [...actions, createActiveTask(app)];
                    }
                    return task.newListr(actions, {
                        rendererOptions: {
                            collapse: true
                        }
                    });
                }
            };
        })
    ];
}
exports.createInstallAppTasks = createInstallAppTasks;
function createExecCommandTasks(ctx) {
    return ctx.tasks
        .filter((task) => (0, utils_1.isCommandType)(task))
        .map((command) => {
        const title = command.description;
        const cmds = Array.isArray(command.cmd) ? command.cmd : [command.cmd];
        return {
            title,
            task: async (_, task) => {
                try {
                    for (const cmd of cmds) {
                        if (isSudoCommand(cmd)) {
                            const password = await task.prompt({
                                type: 'password',
                                name: 'password',
                                message: 'Enter sudo password?'
                            });
                            const { stderr } = (0, lib_1.exec)(cmd.replace('sudo', `echo ${password} | sudo -S`).trim());
                            if (stderr && stderr !== 'Password:') {
                                throw new Error(stderr.toString());
                            }
                        }
                        else {
                            const { code, stderr } = (0, lib_1.exec)(cmd);
                            if (code) {
                                throw new Error(stderr);
                            }
                        }
                    }
                }
                catch (error) {
                    throw new Error(chalk_1.default.red(error.message));
                }
            }
        };
    });
}
exports.createExecCommandTasks = createExecCommandTasks;
function createGenerateTmpDirectoryTask() {
    return async (ctx) => {
        try {
            ctx.tmpdir = await (0, lib_1.tmpdir)();
        }
        catch (error) {
            throw new Error(chalk_1.default.red(error.message));
        }
    };
}
exports.createGenerateTmpDirectoryTask = createGenerateTmpDirectoryTask;
function createRemoveTmpDirectoryTask() {
    return async (ctx) => {
        try {
            if (ctx.tmpdir) {
                await (0, util_1.promisify)(rimraf_1.default)(ctx.tmpdir);
            }
        }
        catch (error) {
            throw new Error(chalk_1.default.red(error.message));
        }
    };
}
exports.createRemoveTmpDirectoryTask = createRemoveTmpDirectoryTask;
function createDownloadTask(app) {
    return {
        title: 'Downloading',
        task: async (ctx, task) => {
            const url = getDownloadUrl(app.downloadUrl);
            if ((0, utils_1.isMasUrl)(url)) {
                task.title = 'Downloaded';
                ctx.filePaths.set(app.name, (0, utils_1.resolveMasUrl)(url));
            }
            else if ((0, lib_1.isBrewUrl)(url)) {
                task.title = 'Downloaded';
                ctx.filePaths.set(app.name, url);
            }
            else {
                try {
                    const filePath = await (0, lib_1.download)({
                        url,
                        dir: ctx.tmpdir,
                        onProgress: ({ percent }) => (task.title = `Downloading [${percent}%]`)
                    });
                    task.title = 'Downloaded';
                    ctx.filePaths.set(app.name, filePath);
                }
                catch (error) {
                    throw new Error(chalk_1.default.red(error.message));
                }
            }
        }
    };
}
function createInstallTask(app) {
    return {
        title: 'Installing',
        task: async (ctx, task) => {
            try {
                const filePath = await (0, lib_1.install)({
                    appName: app.name,
                    dest: getDestDirectory(app.name),
                    filePath: ctx.filePaths.get(app.name)
                });
                task.title = 'Installed';
                ctx.filePaths.set(app.name, filePath);
            }
            catch (error) {
                throw new Error(chalk_1.default.red(error.message));
            }
        }
    };
}
function createActiveTask(app) {
    return {
        title: 'Activating',
        task: async (_, task) => {
            try {
                await (0, lib_1.active)(getActiveActionSteps(app));
                task.title = 'Activated';
            }
            catch (error) {
                throw new Error(chalk_1.default.red(error.message));
            }
        }
    };
}
function createCommandLineToolsTasks() {
    return {
        title: 'CommandLineTools',
        task: (_, task) => {
            const isInstalled = lib_1.commandLineTools.isInstalled();
            return task.newListr([
                {
                    title: 'Downloading',
                    task: (_, task) => {
                        if (!isInstalled) {
                            lib_1.commandLineTools.download();
                        }
                        task.title = 'Downloaded';
                    }
                },
                {
                    title: 'Installing',
                    task: (_, task) => {
                        if (!isInstalled) {
                            lib_1.commandLineTools.install();
                        }
                        task.title = 'Installed';
                    }
                }
            ], {
                rendererOptions: {
                    collapse: true
                }
            });
        }
    };
}
function isSudoCommand(cmd) {
    return cmd.slice(0, 4) === 'sudo';
}
function getDownloadUrl(url) {
    if (Array.isArray(url)) {
        return url.filter((item) => item[0] === (lib_1.isAppleCPU ? 'arm' : 'intel'))[0][1];
    }
    else {
        return url;
    }
}
function getDestDirectory(appName) {
    return appName.toLowerCase() === 'brew' ? lib_1.Homebrew_DIR : lib_1.APP_DIR;
}
function hasActiveAction(action) {
    for (const item of action) {
        if (Array.isArray(item) && item[0] === 'active') {
            return true;
        }
    }
    return false;
}
function getActiveActionSteps(app) {
    for (const item of app.action) {
        if (Array.isArray(item) && item[0] === 'active') {
            return item[1];
        }
    }
    return [];
}
